@@ -1,19 +1,19 @@
 #!/usr/bin/env python
 
-import pathlib
 import paramiko
 import requests
 import sys
 import time
 import threading
-import configparser
 import os
 import logging
+
+from paramiko import SSHException
 from termcolor import colored
 
 logger = logging.getLogger('MasterSSH')
 logger.setLevel(logging.INFO)
-logging.basicConfig(format='[%(asctime)s][%(levelname)s] %(threadName)s: %(message)s', datefmt='%Y/%m/%d %H:%M:%S')
+logging.basicConfig(format='[%(asctime)s] %(threadName)s: %(message)s', datefmt='%Y/%m/%d %H:%M:%S')
 
 
 class MasterSSH:
@@ -110,10 +110,10 @@ class MasterSSH:
 
                 if len(data) == 4:
                     self.credentials.append({
-                        'name': data[0],
-                        'host': data[1],
-                        'username': data[2],
-                        'password': data[3]
+                        'name': data[0].strip(),
+                        'host': data[1].strip(),
+                        'username': data[2].strip(),
+                        'password': data[3].strip()
                     })
 
     """
@@ -122,7 +122,7 @@ class MasterSSH:
 
     def create_connections(self):
         if self.credentials:
-            self.print_message("Connecting to servers...")
+            self.print_message("Connecting to %s servers..." % len(self.credentials))
 
         thread_pool = {}
         thread_pos = 1
@@ -135,14 +135,14 @@ class MasterSSH:
             # If there are specific servers user wants to use, use them ...
             if use_selected_servers:
                 if cred['name'] in self.selected_servers:
-                    thread_pool[thread_pos] = threading.Thread(target=self.connect, name=cred['name'], args=(cred['name'], cred['host'], cred['username'], cred['password'], self.config.port))
+                    thread_pool[thread_pos] = threading.Thread(target=self.connect, name=cred['name'], args=(cred['name'], cred['host'], cred['username'], cred['password'], self.config.port, self.config.connection_timeout))
                     thread_pool[thread_pos].daemon = True
                     thread_pool[thread_pos].start()
                     thread_pos += 1
 
             # ... if not, use all of them
             else:
-                thread_pool[thread_pos] = threading.Thread(target=self.connect, name=cred['name'], args=(cred['name'], cred['host'], cred['username'], cred['password'], self.config.port))
+                thread_pool[thread_pos] = threading.Thread(target=self.connect, name=cred['name'], args=(cred['name'], cred['host'], cred['username'], cred['password'], self.config.port, self.config.connection_timeout))
                 thread_pool[thread_pos].daemon = True
                 thread_pool[thread_pos].start()
                 thread_pos += 1
@@ -156,29 +156,25 @@ class MasterSSH:
     Connect to the server
     """
 
-    def connect(self, name, host, username, password, port):
+    def connect(self, name, host, username, password, port, timeout):
         tries = 1
         max_tries = self.config.max_retries
 
-        while True:
-            if tries == max_tries:
-                self.print_error("Unable to connect, giving up!")
-                break
-
+        while tries <= max_tries:
             try:
                 client = paramiko.SSHClient()
                 client.set_log_channel('SSHClient')
                 client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-                client.connect(host, username=username, password=password, port=port)
+                client.connect(host, username=username, password=password, port=port, timeout=timeout)
                 client.get_transport().set_keepalive(30)
 
                 self.connection_pool[name] = client
                 self.print_success("Successfully connected!")
 
-                break
+                return
             except paramiko.ssh_exception.AuthenticationException:
                 self.print_error("Failed to connect - Wrong login details!")
-                break
+                return
             except Exception as e:
                 self.print_error("Failed to connect! Retrying (%s/%s)" % (tries, max_tries))
                 self.print_exception(e)
@@ -186,6 +182,8 @@ class MasterSSH:
                 tries += 1
                 time.sleep(self.config.retry_delay)
 
+        self.print_error("Unable to connect, giving up!")
+
     """
     Create connection closing threads
     """
@@ -228,7 +226,11 @@ class MasterSSH:
 
         while True:
             # Listen for user input
-            cmd = input('master-ssh$ ').strip()
+            try:
+                cmd = input('master-ssh$ ').strip()
+            except KeyboardInterrupt:
+                self.close_connections()
+                break
 
             if cmd == "":
                 self.print_error('Please enter your command!')
@@ -245,31 +247,20 @@ class MasterSSH:
                     """
 
                     if cmd == "#help" or cmd == "#":
-
-                        print("Welcome to master-ssh!")
-                        print("Made by Jaan Porkon <jaantrill[at]gmail.com>")
-                        print("https://github.com/JaanPorkon/master-ssh")
-                        print("")
-                        print("Example syntax:")
-                        print("#[command]")
-                        print("")
-                        print("Example command:")
-                        print("#help")
-                        print("")
-                        print("Below you can find available commands that you can use:")
-                        print("")
-                        print("help                                        - Displays this message")
-                        print("list-connections                            - Lists the state of all connections")
-                        print("disconnect:[hostname,hostname2,...]         - Disconnects a specific host and removes it from the pool")
-                        print("                                            - Example:")
-                        print("                                              #disconnect:host1")
-                        print("                                            - or multiple servers:")
-                        print("                                              #disconnect:host2,host3")
-                        print("ignore:[hostname,hostname2] [command]       - Ignores servers that are listed")
-                        print("connect:[host],[username],[password],[name] - Connects to a server. [name] is optional, but suggested")
+                        print('{: <5}'.format('Welcome to MasterSSH!'))
+                        print('Made by Jaan Porkon')
+                        print('https://github.com/JaanPorkon/master-ssh')
+                        print('')
+                        print('Examples:')
+                        print('')
+                        print('{param: <55}{help}'.format(param='#list', help='Lists all connections.'))
+                        print('{param: <55}{help}'.format(param='#connect [host] [user] [password] [port] [name]', help='Connects to a new server. Port and name are optional!'))
+                        print('{param: <55}{help}'.format(param='#disconnect [host]', help='Disconnects the host.'))
+                        print('{param: <55}{help}'.format(param='#ignore [host,host2,...] [command]', help='Does not send a command to ignored servers.'))
+                        print('')
 
                     # List all connections
-                    elif cmd == "#list-connections":
+                    elif cmd == "#list":
                         for name, connection in self.connection_pool.items():
                             is_alive = connection.get_transport().is_active()
 
@@ -278,27 +269,30 @@ class MasterSSH:
                             else:
                                 self.print_server_error("%s is disconnected!" % name)
 
-                    elif cmd.startswith('#connect:'):
-                        data = cmd.replace('#connect:', '').split(':')
+                    elif cmd.startswith('#connect'):
+                        data = cmd.replace('#connect', '').strip().split(' ')
+                        data_length = len(data)
 
-                        # host:username:password:name*
-                        # * name is optional
-
-                        if len(data) == 3:
-                            self.connect(data[0], data[0], data[1], data[2])
-                        elif len(data) == 4:
-                            self.connect(data[3], data[0], data[1], data[2])
+                        if data_length < 3:
+                            self.print_error('Not enough arguments! Use: #connect [host] [user] [password] [port] [name] - port and name are optional!')
                         else:
-                            self.print_error("Not enough arguments!")
+                            fields = {'host': data[0], 'user': data[1], 'password': data[2], 'port': int(self.config.port), 'name': data[0]}
+
+                            if data_length == 4:
+                                fields['port'] = int(data[3])
+                            elif data_length == 5:
+                                fields['port'] = int(data[3])
+                                fields['name'] = data[4]
+
+                            self.connect(fields['name'], fields['host'], fields['user'], fields['password'], fields['port'])
 
                     # Disconnect specific servers from the pool
-                    elif cmd.startswith('#disconnect:'):
-
-                        disconnected = False
-                        servers = cmd.strip().split(':')[-1]
+                    elif cmd.startswith('#disconnect'):
+                        servers = cmd.replace('#disconnect', '').strip()
 
                         for server in servers.split(','):
                             if server in self.connection_pool:
+                                server = server.strip()
                                 connection = self.connection_pool[server]
 
                                 try:
@@ -307,19 +301,18 @@ class MasterSSH:
                                         connection.close()
 
                                         self.connection_pool.pop(server)
-                                        self.print_success(server + " successfully disconnected!")
+                                        self.print_success("%s successfully disconnected!" % server)
                                     else:
                                         self.connection_pool.pop(server)
 
-                                    disconnected = True
-
                                 except Exception as e:
-                                    self.print_error('Unable to disconnect: ' + server + ' (' + str(e) + ')')
+                                    self.print_error('Unable to disconnect: %s!' % server)
+                                    self.print_exception(e)
 
                             else:
-                                self.print_error(server + " does not exist!")
+                                self.print_error("%s does not exist!" % server)
 
-                        if len(self.connection_pool) == 0 and disconnected:
+                        if len(self.connection_pool) == 0:
                             self.print_message("Connection pool is empty, closing the program..")
                             sys.exit(0)
 
@@ -329,84 +322,36 @@ class MasterSSH:
                         SERVER COMMANDS
                         """
 
-                        use_ignore_list = False
-                        server_list_start = 1
-
-                        find_space = cmd.find(' ')
-                        command = cmd[(find_space + 1):len(cmd)].strip()
-
                         # If user wishes to ignore specific servers, do so ...
-                        if cmd.startswith("#ignore:"):
-                            server_list_start = 8
-                            use_ignore_list = True
+                        use_ignore_list = cmd.startswith("#ignore")
+                        ignored_server_list = []
 
-                        server = cmd[server_list_start:find_space].strip()
+                        servers, command = cmd.replace('#ignore ', '').split(' ', 1)
+
+                        for server in servers.split(','):
+                            ignored_server_list.append(server)
+
+                        thread_pool = {}
+                        thread_pos = 1
 
                         if use_ignore_list:
-                            ignore_list = server.strip().split(',')
-
-                            thread_pool = {}
-                            thread_pos = 1
-
                             for name, connection in self.connection_pool.items():
-                                if name not in ignore_list:
-                                    thread_pool[thread_pos] = threading.Thread(target=self.execute, name=name, args=(command, name, connection,))
+                                if name not in ignored_server_list:
+                                    thread_pool[thread_pos] = threading.Thread(target=self.execute, name=name, args=(command.strip(), connection, int(self.config.cmd_timeout)))
                                     thread_pool[thread_pos].daemon = True
                                     thread_pool[thread_pos].start()
                                     thread_pos += 1
-
-                            for i in range(1, thread_pos):
-                                thread_pool[i].join()
-
-                            ignore_list = []
-
-                        # ... if not, send the command to ...
                         else:
+                            for name, connection in self.connection_pool.items():
+                                thread_pool[thread_pos] = threading.Thread(target=self.execute, name=name, args=(command.strip(), connection, int(self.config.cmd_timeout)))
+                                thread_pool[thread_pos].daemon = True
+                                thread_pool[thread_pos].start()
+                                thread_pos += 1
 
-                            # ... specific servers that user has defined or ...
-                            if "," in server:
+                        for i in range(1, thread_pos):
+                            thread_pool[i].join()
 
-                                thread_pool = {}
-                                thread_pos = 1
-
-                                for server in server.split(','):
-
-                                    if server not in self.connection_pool:
-                                        self.print_error("%s does not exist!" % server)
-                                    else:
-                                        connection = self.connection_pool[server]
-
-                                        thread_pool[thread_pos] = threading.Thread(target=self.execute, name=server, args=(command, server, connection,))
-                                        thread_pool[thread_pos].daemon = True
-                                        thread_pool[thread_pos].start()
-                                        thread_pos += 1
-
-                                for i in range(1, thread_pos):
-                                    thread_pool[i].join()
-
-                            # ... send the command to one specific server
-                            else:
-
-                                if server not in self.connection_pool:
-                                    self.print_error("%s does not exist!" % server)
-                                else:
-                                    stdin, stdout, stderr = self.connection_pool[server].exec_command(command)
-
-                                    error = ""
-
-                                    for line in stderr:
-                                        error += "\n" + line
-
-                                    self.print_server_error(error.strip())
-
-                                    response = ""
-
-                                    for line in stdout:
-                                        response += line
-
-                                    self.print_server_action(response.strip())
-
-                # If user haven't defined any internal comments, send user's command to all of the servers
+                # If user haven't defined any internal commands, send user's command to all of the servers
                 else:
                     if len(self.connection_pool) == 0:
                         self.print_error('There are no active connections!')
@@ -415,7 +360,7 @@ class MasterSSH:
                         thread_pos = 1
 
                         for name, connection in self.connection_pool.items():
-                            thread_pool[thread_pos] = threading.Thread(target=self.execute, name=name, args=(cmd, name, connection,))
+                            thread_pool[thread_pos] = threading.Thread(target=self.execute, name=name, args=(cmd, connection, int(self.config.cmd_timeout)))
                             thread_pool[thread_pos].daemon = True
                             thread_pool[thread_pos].start()
                             thread_pos += 1
@@ -427,28 +372,35 @@ class MasterSSH:
     Execute user's command
     """
 
-    def execute(self, cmd, name, connection):
-        stdin, stdout, stderr = connection.exec_command(cmd)
+    def execute(self, cmd, connection, timeout):
+        try:
+            stdin, stdout, stderr = connection.exec_command(cmd, timeout=timeout)
 
-        error = ""
+            error = ""
 
-        for line in stderr:
-            error += "\n%s" % line
+            for line in stderr:
+                error += '%s\n' % line.strip()
 
-        error = error.strip()
+            error = error.strip()
 
-        if error != "":
-            self.print_server_error(error.strip())
+            if error != "":
+                self.print_server_error(error)
 
-        response = ""
+            response = ""
 
-        for line in stdout:
-            response += "\n%s" % line
+            for line in stdout:
+                response += '%s\n' % line.strip()
 
-        response = response.strip()
+            response = response.strip()
 
-        if response != "":
-            self.print_server_action(response.strip())
+            if response != "":
+                self.print_server_action(response)
+        except SSHException as e:
+            self.print_error('Unable to execute the command!')
+            self.print_exception(e)
+        except Exception as e:
+            self.print_error('Something went wrong!')
+            self.print_exception(e)
 
     """
     Helper methods
@@ -460,17 +412,17 @@ class MasterSSH:
     def print_action(self, message):
         logger.info(colored(message, color='green'))
 
-    def print_server_action(self, message):
-        logger.info('\n%s' % colored(message, color='green'))
-
     def print_success(self, message):
         logger.info(colored(message, color='green'))
 
     def print_error(self, message):
-        logger.error('master-ssh: %s' % colored(message, on_color='on_red', attrs=['bold']))
-
-    def print_server_error(self, message):
         logger.error(colored(message, on_color='on_red', attrs=['bold']))
 
     def print_exception(self, err):
-        logger.exception(err)
\ No newline at end of file
+        logger.exception(err)
+
+    def print_server_action(self, message):
+        logger.info('\n%s' % colored(message, color='green'))
+
+    def print_server_error(self, message):
+        logger.error('\n%s' % colored(message, on_color='on_red', attrs=['bold']))
\ No newline at end of file
